"""
    type specific test based on spoke base tests
"""
from wheelcms_axle.tests.test_spoke import BaseSpokeTest, BaseSpokeTemplateTest
from wheelcms_spokes.page import PageType, Page
from wheelcms_axle.tests.test_impexp import BaseSpokeImportExportTest
from wheelcms_axle.tests.test_search import BaseTestSearch

class TestPageSpokeTemplate(BaseSpokeTemplateTest):
    """ Test the Page type """
    type = PageType

    def valid_data(self):
        """ return additional data for Page validation """
        return dict(body="Hello World")


class TestPageSpoke(BaseSpokeTest):
    """ Test the Page type """
    type = PageType

class TestPageSpokeImpExp(BaseSpokeImportExportTest):
    type = Page
    spoke = PageType

class TestPageSpokeSearch(BaseTestSearch):
    type = PageType



from django.db import models

from wheelcms_axle.models import type_registry
from wheelcms_axle.templates import template_registry
from wheelcms_axle.content import FileContent
from wheelcms_axle.spoke import FileSpoke

class File(FileContent):
    """ Holds a file """
    ## cannot be named file - that's used for the content base relation
    storage = models.FileField(upload_to="files", blank=False)


## hide filename from form - extract it from uploaded file.
##
## provide list of mimetypes for mimetype, provide detection option

class FileType(FileSpoke):
    model = File

    title = "A file"
    children = ()

    type_icon = icon = "file.png"

    def detail_template(self):
        """ A small detail template, used in browse modal """
        return "wheelcms_spokes/file_detail.html"


template_registry.register(FileType, "wheelcms_spokes/file_view.html", "Basic File view", default=True)
type_registry.register(FileType)



"""
    test file/image based on spoke base tests
"""

import pytest

from wheelcms_axle.tests.test_spoke import BaseSpokeTemplateTest, \
                                             BaseSpokeTest
from wheelcms_spokes.file import FileType, File
from wheelcms_spokes.image import ImageType, Image
from django.core.files.uploadedfile import SimpleUploadedFile
from wheelcms_axle.tests.test_impexp import BaseSpokeImportExportTest
from wheelcms_axle.tests.test_search import BaseTestSearch


class BaseImageFileTemplateTest(BaseSpokeTemplateTest):
    """
        Shared customization/tests
    """
    def valid_files(self):
        """ return an image, will work for both file and image uploads """
        return dict(storage=SimpleUploadedFile("foo.png", 
                    'GIF87a\x01\x00\x01\x00\x80\x01\x00\x00\x00\x00ccc,\x00'
                    '\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02D\x01\x00;'))

    def test_children_restriction(self, client):
        """ by default, a file or image can't have kids """
        assert self.type.children is not None
        assert len(self.type.children) == 0

class BaseImageFileTest(BaseSpokeTest):
    def test_download(self, client):
        data = 'GIF87a\x01\x00\x01\x00\x80\x01\x00\x00\x00\x00ccc,\x00' \
                   '\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02D\x01\x00;'
        storage=SimpleUploadedFile("foo.png", data)
        f = self.type.model(storage=storage,
                            filename="bar.png",
                            content_type="application/octet-stream").save()
        spoke = self.type(f)
        response = spoke.download(None, None, 'download')
        assert response.content == data
        assert response.has_header('Content-Disposition')
        assert response['Content-Disposition'] == \
               'attachment; filename=bar.png'
        assert response['Content-Type'] == "application/octet-stream"

    def test_download_defaults(self, client):
        data = 'GIF87a\x01\x00\x01\x00\x80\x01\x00\x00\x00\x00ccc,\x00' \
                   '\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02D\x01\x00;'
        storage=SimpleUploadedFile("foo.png", data)
        f = self.type.model(storage=storage).save()
        spoke = self.type(f)
        response = spoke.download(None, None, 'download')
        assert response.content == data
        assert response.has_header('Content-Disposition')
        assert response['Content-Disposition'] == \
               'attachment; filename=foo.png'
        assert response['Content-Type'] == "image/png"

    def test_filename_slash(self, client):
        """ make sure the filename cannot contain directory components """
        data = 'GIF87a\x01\x00\x01\x00\x80\x01\x00\x00\x00\x00ccc,\x00' \
                   '\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02D\x01\x00;'
        storage=SimpleUploadedFile("foo.png", data)
        f = self.type.model(storage=storage, filename="/etc/passwd").save()
        assert f.filename == "passwd"

        f = self.type.model(storage=storage, filename="../foo.png").save()
        assert f.filename == "foo.png"

    def test_download_state(self, client):
        """ file must be "visible" in order to be downloadable """
        pytest.skip("TODO XXX")

class TestImageSpokeTemplate(BaseImageFileTemplateTest):
    """
        Test the image spoke
    """
    type = ImageType


class TestImageSpoke(BaseImageFileTest):
    """
        Test the image spoke
    """
    type = ImageType


class TestImageSpokeImpExp(BaseSpokeImportExportTest):
    type = Image
    spoke = ImageType

class TestImageSpokeSearch(BaseTestSearch):
    type = ImageType

class TestFileSpokeTemplate(BaseImageFileTemplateTest):
    """
        Test the file spoke
    """
    type = FileType


class TestFileSpoke(BaseImageFileTest):
    """
        Test the file spoke
    """
    type = FileType


class TestImageFileImpExp(BaseSpokeImportExportTest):
    type = File
    spoke = FileType

    def create(self, **kw):
        f = SimpleUploadedFile("foo.png", 
                    'GIF87a\x01\x00\x01\x00\x80\x01\x00\x00\x00\x00ccc,\x00'
                    '\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02D\x01\x00;')
        t = self.type(storage=f, **kw).save()
        tt = self.spoke(t)
        return tt

    def test_capable_serialize(self, client):
        # import pytest; pytest.set_trace()
        super(TestImageFileImpExp, self).test_capable_serialize(client)

class TestFileSpokeSearch(BaseTestSearch):
    type = FileType



from django.db import models
from django import forms

from django.utils.html import strip_tags
from django.utils.text import Truncator

from wheelcms_axle.content import type_registry
from wheelcms_axle.content import Content
from wheelcms_axle.templates import template_registry
from wheelcms_axle.spoke import Spoke, indexfactory, SpokeCharField
from wheelcms_axle.node import Node
from wheelcms_axle.node import node_proxy_factory

from wheelcms_axle.forms import formfactory
from wheelcms_axle import access
from wheelcms_axle.utils import get_active_language

from wheelcms_axle.forms import TinyMCE
from tinymce.models import HTMLField

##
## Either HTMLField or TinyMCE widget. But a demonstration on how to alter
## a Spoke's form is also nice to have.

class PageBase(Content):
    """ A simple page object """

    class Meta:
        abstract = True

    body = models.TextField(blank=False)
    # body = HTMLField(blank=False)

class Page(PageBase):
    pass

class PageForm(formfactory(Page)):
    body = forms.CharField(widget=TinyMCE(), required=False)


class PageType(Spoke):
    document_fields = Spoke.document_fields + ("body", )

    model = Page
    title = "A page"
    form = PageForm

    @property
    def icon(self):
        if self.instance.node.children().exists():
            return "folder_page.png"
        return "page.png"

    def index_description(self):
        """ truncate body text if no explicit description available """
        return self.description()

    def description(self):
        """ truncate body text if no explicit description available """
        if self.instance.description:
            return self.instance.description

        return Truncator(strip_tags(self.instance.body)).words(50,
                         truncate=" ...")

    @classmethod
    def index(cls):
        class PageIndex(indexfactory(cls)):
            description = SpokeCharField(spoke=cls,
                                         stored=True, indexed=False,
                                         model_attr="index_description")

        return PageIndex

    def context(self, handler, request, node):
        ctx = super(PageType, self).context(handler, request, node)
        wpm = 180  # avg adult for monitor
        # XXX NOTTESTED
        ctx['reading_minutes'] = (len(strip_tags(self.instance.body).split())/wpm) + 1
        return ctx

def contentlisting_context(handler, request, node):
    language = get_active_language(request)

    q = node_proxy_factory(Node, language).objects.children(node).filter(contentbase__language=language).order_by("position")

    if not access.has_access(request.user, node):
        q = q.public()

    return dict(contents=q)

type_registry.register(PageType)
template_registry.register(PageType, "wheelcms_spokes/page_view.html",
                           "Basic Page view", default=True)
template_registry.register(PageType, "wheelcms_spokes/page_contents.html",
                           "Contents Listing", default=False,
                           context=contentlisting_context)



from django.contrib import admin
from wheelcms_spokes import models
from wheelcms_axle.models import type_registry

## create an admin for all registered types

for spoke in type_registry.values():
    class SpokeAdmin(admin.ModelAdmin):
        model = spoke.model
    admin.site.register(spoke.model, SpokeAdmin)



from django.db import models
from django import forms

from django.utils.html import strip_tags
from django.utils.text import Truncator

from tinymce.widgets import TinyMCE

from wheelcms_axle.content import type_registry, Content
from wheelcms_axle.templates import template_registry
from wheelcms_axle.spoke import Spoke
from wheelcms_axle.forms import formfactory


class NewsBase(Content):
    """ A news object """
    class Meta:
        abstract = True

    intro = models.TextField(blank=False)
    body = models.TextField(blank=False)

class News(NewsBase):
    pass

class NewsForm(formfactory(News)):
    body = forms.CharField(widget=TinyMCE(), required=False)

class NewsType(Spoke):
    model = News
    form = NewsForm

    title = "A simple News item"

    type_icon = icon = "news.png"

    def index_description(self):
        """ truncate body text if no explicit description available """
        return self.description()

    def description(self):
        """ truncate body text if no explicit description available """
        if self.instance.intro:
            return self.instance.intro

        if self.instance.description:
            return self.instance.description

        return Truncator(strip_tags(self.instance.body)).words(50,
                         truncate=" ...")
template_registry.register(NewsType, "wheelcms_spokes/news_view.html", "Basic News view", default=True)
type_registry.register(NewsType)



from django.db import models
from wheelcms_axle.content import type_registry, ImageContent
from wheelcms_axle.templates import template_registry
from wheelcms_axle.spoke import FileSpoke


class Image(ImageContent):
    """ Holds an image.  """
    ## cannot be named image - that's used for the content base relation
    storage = models.ImageField(upload_to="images", blank=False)

class ImageType(FileSpoke):
    model = Image

    title = "An image"
    children = ()

    default_language = 'any'  # images are usually not language specific

    type_icon = icon = "image.png"

    def detail_template(self):
        """ A small detail template, used in browse modal """
        return "wheelcms_spokes/image_detail.html"

template_registry.register(ImageType, "wheelcms_spokes/image_view.html", "Basic Image view", default=True)
type_registry.register(ImageType)



"""
    type specific test based on spoke base tests
"""
from wheelcms_axle.tests.test_spoke import BaseSpokeTest, BaseSpokeTemplateTest
from wheelcms_spokes.news import NewsType, News
from wheelcms_axle.tests.test_impexp import BaseSpokeImportExportTest
from wheelcms_axle.tests.test_search import BaseTestSearch

class TestNewsSpokeTemplate(BaseSpokeTemplateTest):
    """ Test the News type """
    type = NewsType

    def valid_data(self):
        """ return additional data for News validation """
        return dict(intro="Hi There!", body="Hello World")

class TestNewsSpoke(BaseSpokeTest):
    """ Test the News type """
    type = NewsType

class TestNewsSpokeImpExp(BaseSpokeImportExportTest):
    type = News
    spoke = NewsType

class TestNewsSpokeSearch(BaseTestSearch):
    type = NewsType
